@@ -6,6 +6,8 @@
  */
 
 #include "unikey-im.h"
+#include "usrkeymap.h"
+#include "vnconv.h"
 #include <fcitx-utils/charutils.h>
 #include <fcitx-utils/standardpath.h>
 #include <fcitx-utils/utf8.h>
@@ -15,7 +17,7 @@
 #include <fcitx/menu.h>
 #include <fcitx/statusarea.h>
 #include <fcitx/userinterfacemanager.h>
-#include <vnconv.h>
+#include <fcntl.h>
 
 namespace fcitx {
 
@@ -500,10 +502,25 @@ void UnikeyEngine::populateConfig() {
 
 void UnikeyEngine::reloadConfig() {
     readAsIni(config_, "conf/unikey.conf");
+    reloadKeymap();
     populateConfig();
     reloadMacroTable();
 }
 
+void UnikeyEngine::reloadKeymap() {
+    // Keymap need to be reloaded before populateConfig.
+    auto keymapFile = StandardPath::global().open(
+        StandardPath::Type::PkgConfig, "unikey/keymap.txt", O_RDONLY);
+    if (keymapFile.isValid()) {
+        UniqueFilePtr fp = fs::openFD(keymapFile, "rb");
+
+        UkLoadKeyMap(fp.get(), im_.sharedMem()->usrKeyMap);
+        im_.sharedMem()->usrKeyMapLoaded = true;
+    } else {
+        im_.sharedMem()->usrKeyMapLoaded = false;
+    }
+}
+
 void UnikeyEngine::save() {}
 
 std::string UnikeyEngine::subMode(const InputMethodEntry &, InputContext &) {
